using System;
using System.Collections.Generic;
using System.Text;
using Westwind.InternetTools;
using System.Web;
using Westwind.Utilities;
using Westwind.Web.JsonSerializers;
using System.Net;
using System.Web.Script.Serialization;
using System.Text.RegularExpressions;

namespace Westwind.Globalization
{
    /// <summary>
    /// Provides basic translation features via several Web interfaces
    /// 
    /// NOTE: These services may change their format or otherwise fail.
    /// </summary>
    public class TranslationServices
    {
        /// <summary>
        /// Error message set when an error occurs in the translation service
        /// </summary>
        public string ErrorMessage
        {
            get { return _ErrorMessage; }
            set { _ErrorMessage = value; }
        }
        private string _ErrorMessage = "";

        /// <summary>
        /// Timeout for how long to wait for a translation
        /// </summary>
        public int TimeoutSeconds
        {
            get { return _TimeoutSeconds; }
            set { _TimeoutSeconds = value; }
        }
        private int _TimeoutSeconds = 10;


        /// <summary>
        /// Translates a string into another language using Google's translate API JSON calls.
        /// <seealso>Class TranslationServices</seealso>
        /// </summary>
        /// <param name="Text">Text to translate. Should be a single word or sentence.</param>
        /// <param name="FromCulture">
        /// Two letter culture (en of en-us, fr of fr-ca, de of de-ch)
        /// </param>
        /// <param name="ToCulture">
        /// Two letter culture (as for FromCulture)
        /// </param>
        public string TranslateGoogle(string text, string fromCulture, string toCulture)
        {
            fromCulture = fromCulture.ToLower();
            toCulture = toCulture.ToLower();

            // normalize the culture in case something like en-us was passed 
            // retrieve only en since Google doesn't support sub-locales
            string[] tokens = fromCulture.Split('-');
            if (tokens.Length > 1)
                fromCulture = tokens[0];
            
            // normalize ToCulture
            tokens = toCulture.Split('-');
            if (tokens.Length > 1)
                toCulture = tokens[0];
            
            string url = string.Format(@"http://translate.google.com/translate_a/t?client=j&text={0}&hl=en&sl={1}&tl={2}",                                     
                                       HttpUtility.UrlEncode(text),fromCulture,toCulture);

            // Retrieve Translation with HTTP GET call
            string html = null;
            try
            {
                WebClient web = new WebClient();

                // MUST add a known browser user agent or else response encoding doen't return UTF-8 (WTF Google?)
                web.Headers.Add(HttpRequestHeader.UserAgent, "Mozilla/5.0");
                web.Headers.Add(HttpRequestHeader.AcceptCharset, "UTF-8");

                // Make sure we have response encoding to UTF-8
                web.Encoding = Encoding.UTF8;
                html = web.DownloadString(url);
            }
            catch (Exception ex)
            {
                this.ErrorMessage = Westwind.Globalization.Resources.Resources.ConnectionFailed + ": " +
                                    ex.GetBaseException().Message;
                return null;
            }

            // Extract out trans":"...[Extracted]...","from the JSON string
            string result = Regex.Match(html, "trans\":(\".*?\"),\"", RegexOptions.IgnoreCase).Groups[1].Value;            

            if (string.IsNullOrEmpty(result))
            {
                this.ErrorMessage = Westwind.Globalization.Resources.Resources.InvalidSearchResult;
                return null;
            }

            // result string must be JSON decoded
            return WebUtils.DecodeJsString(result);

            // Result is a JavaScript string so we need to deserialize it properly
            //JavaScriptSerializer ser = new JavaScriptSerializer();
            //return ser.Deserialize(result, typeof(string)) as string;            
        }

        /// <summary>
        /// Translates a string using Yahoo's Babel fish service.
        /// 
        /// &lt;&lt;a href=&quot;http://babelfish.yahoo.com/&quot; 
        /// target=&quot;top&quot;&gt;&gt;http://babelfish.yahoo.com/&lt;&lt;/a&gt;&gt;
        /// <seealso>Class TranslationServices</seealso>
        /// </summary>
        /// <param name="Text">
        /// The text to translate
        /// </param>
        /// <param name="FromCulture">
        /// Two letter culture (en of en-us, fr of fr-ca, de of de-ch)
        /// </param>
        /// <param name="ToCulture">
        /// Two letter culture
        /// </param>
        public string TranslateBabelFish(string Text, string FromCulture, string ToCulture)
        {
            FromCulture = FromCulture.ToLower();
            ToCulture = ToCulture.ToLower();

            string[] Tokens = FromCulture.Split('-');
            if (Tokens.Length > 1)
                FromCulture = Tokens[0];

            Tokens = ToCulture.Split('-');
            if (Tokens.Length > 1)
                ToCulture = Tokens[0];

            string LangPair = FromCulture + "_" + ToCulture;

            HttpClient Http = new HttpClient();

            // ei=UTF-8&fr=bf-badge&trurl=&trtext=Hello&lp=en_de&submit=Translate
            Http.AddPostKey("ei", "UTF-8");
            Http.AddPostKey("fr", "bf-badge");
            Http.AddPostKey("trtext", Text);
            Http.AddPostKey("lp", LangPair);
            Http.AddPostKey("submit", "Translate");

            string Html = Http.DownloadString("http://babelfish.yahoo.com/translate_txt");
            if (string.IsNullOrEmpty(Html))
            {
                this.ErrorMessage = Http.ErrorMessage;
                return null;
            }
            
            // <div id="result"><div style="padding:0.6em;">Hallo</div></div>
            string Result = StringUtils.ExtractString(Html, "<div id=\"result\">", "</div>");
            if (Result == "")
            {
                this.ErrorMessage = "Invalid search result. Couldn't find marker.";
                return null;
            }
            Result = Result.Substring(Result.LastIndexOf(">") + 1);

            return HttpUtility.HtmlDecode(Result); 
        }


    }
}
