﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.IO;
using System.Drawing.Imaging;
using System.Web.UI.DataVisualization.Charting;

namespace Westwind.WebToolkit
{
    
    /// <summary>
    /// Wrapper class around ZedGraph to provide some simple graph renderings
    /// </summary>
    public class Graph
    {
        /// <summary>
        /// Allows you to specify data for each series as dictionary of label/value options
        /// along with configuration data for each series 
        /// </summary>
        public List<SeriesData> SeriesList
        {
            get { return _SeriesList; }
            set { _SeriesList = value; }
        }
        private List<SeriesData> _SeriesList = new List<SeriesData>();

        /// <summary>
        /// The type of chart to display
        /// </summary>
        public SeriesChartType ChartType
        {
            get { return _ChartType; }
            set { _ChartType = value; }
        }
        private SeriesChartType _ChartType = SeriesChartType.Line;
        
    
        /// <summary>
        /// Title of the graphTitle
        /// </summary>
        public string GraphTitle
        {
            get { return _GraphTitle; }
            set { _GraphTitle = value; }
        }
        private string _GraphTitle = "";

        /// <summary>
        /// The main chart area background color
        /// </summary>
        public Color BackgroundColor
        {
            get { return _BackgroundColor; }
            set { _BackgroundColor = value; }
        }
        private Color _BackgroundColor = Color.Khaki;

        /// <summary>
        /// The color of the border around the chart.
        /// If Color.Empty is used no border is drawn
        /// </summary>
        public Color BorderColor
        {
            get { return _BorderColor; }
            set { _BorderColor = value; }
        }
        private Color _BorderColor = Color.Black;
        

        /// <summary>
        /// Title for X Axis
        /// </summary>
        public string XAxisTitle
        {
            get { return _XAxisTitle; }
            set { _XAxisTitle = value; }
        }
        private string _XAxisTitle = "";

        /// <summary>
        /// Title for the Y Axis
        /// </summary>
        public string YAxisTitle
        {
            get { return _YAxisTitle; }
            set { _YAxisTitle = value; }
        }
        private string _YAxisTitle = "";

        /// <summary>
        /// Font used for labels and axis labels
        /// </summary>
        public Font LabelFont
        {
            get { return _LabelFont; }
            set { _LabelFont = value; }            
        }
        private Font _LabelFont = new Font("Tahoma", 8.0f, FontStyle.Regular);

        
        /// <summary>
        /// Height of the chart in pixels
        /// </summary>
        public int ChartHeight
        {
            get { return _ChartHeight; }
            set { _ChartHeight = value; }
        }
        private int _ChartHeight = 400;

        /// <summary>
        /// The width of the full chart in pixels
        /// </summary>        
        public int ChartWidth
        {
            get { return _ChartWidth; }
            set { _ChartWidth = value; }
        }
        private int _ChartWidth = 600;

        /// <summary>
        /// If true renders the chart area in 3D
        /// </summary>
        public bool Enable3DChart { get; set; }

        public int LineWidth
        {
            get { return _LineWidth; }
            set { _LineWidth = value; }
        }
        private int _LineWidth = 8;
        
        
        /// <summary>
        /// Draws simple charts using System.Web.DataVisualizations
        /// and returns them as a stream.
        /// 
        /// Image is returned as PNG image stream
        /// </summary>
        /// <param name="fileName">Optional - if provided output is written to file and method returns null</param>
        /// <returns></returns>
        public MemoryStream DrawChart()
        {
            Chart chart = DrawChart_Internal();
    
            var returnStream = new MemoryStream();
            chart.ImageType = ChartImageType.Png;
            chart.SaveImage(returnStream);
            returnStream.Position = 0;
            return returnStream;    
        }

        /// <summary>
        /// Saves an image to a file. File format of the image is
        /// a PNG file.
        /// </summary>
        /// <param name="outputFileName">Filename to write a chart to</param>
        /// <returns></returns>
        public void DrawChart(string outputFileName)
        {
            Chart chart = DrawChart_Internal();
            chart.SaveImage(outputFileName);
        }

        /// <summary>
        /// Draws simple charts using System.Web.DataVisualizations
        /// and returns them as a stream.
        /// </summary>
        /// <param name="fileName">Optional - if provided output is written to file and method returns null</param>
        /// <returns></returns>
        protected Chart DrawChart_Internal()
        {                        
            var chart = new Chart();

            var area = new ChartArea();
            if (Enable3DChart)
                area.Area3DStyle = new ChartArea3DStyle() { Enable3D = true };

            chart.ChartAreas.Add(area);

            // if legegnds are defined - add to chart 
            if (this.SeriesList.Where(set => set.Legend != null).Count() > 0 )            
                chart.Legends.Add(new Legend() { Docking = Docking.Bottom, Name = "Legend" });

            area.BackGradientStyle = GradientStyle.TopBottom;
            area.BackColor = Color.SteelBlue;
            area.BackSecondaryColor = Color.White;

            foreach (SeriesData set in SeriesList)
            {
                // create and customize your data series.
                var series = new Series();
                
                foreach (var item in set.Data)
                {
                    series.Points.AddXY(item.Key, item.Value);
                }                
                
                series.ShadowOffset = Math.Max(LineWidth/3,2);
                series.ShadowColor = Color.Black;

                series.Color = set.LineColor;
                if (series.Color == Color.Empty)
                    series.Color = GetRandomColor();

                series.AxisLabel = set.Legend;                

                series.ChartType = ChartType;                
                series.BorderWidth = LineWidth;
                if (!string.IsNullOrEmpty(set.LabelExpression))
                    series.Label = set.LabelExpression;
                                
                //series.Label = "#VALY";//#ValX:Val";
                //series.LabelFormat = "n2";

                if (!string.IsNullOrEmpty(set.Legend))
                {
                    series.Legend = "Legend";
                    series.LegendText = set.Legend;
                }
                
                //chart.Legends.Add( new LegendItem() { Name=set.Legend, SeriesName=set.Legend })
                chart.Series.Add(series);
            }

            if (!string.IsNullOrEmpty(GraphTitle))
                chart.Titles.Add(GraphTitle);

            chart.Width = ChartWidth;
            chart.Height = ChartHeight;
            
            chart.BackColor = BackgroundColor;
            chart.BackSecondaryColor = Color.White;
            chart.BackGradientStyle = GradientStyle.TopBottom;

            if (BorderColor == Color.Empty)
            {                
                chart.BorderlineWidth = 0;
            }
            else
            {
                chart.BorderlineColor = BorderColor;
                chart.BorderlineWidth = 2;
            }

            return chart;
        }


        private static Array knownColors = null;
        private static Random rnd = null;

        /// <summary>
        /// Retrieves a random color from known color values.
        /// Used for large numbers of graph data lines
        /// </summary>
        /// <returns></returns>
        public static Color GetRandomColor()
        {
            KnownColor col = KnownColor.Transparent;

            if (knownColors == null)
            {
                knownColors = Enum.GetValues(typeof(KnownColor));
                rnd = new Random();
            }
            col = (KnownColor) knownColors.GetValue(rnd.Next(knownColors.Length));
            return Color.FromKnownColor(col);
        }

    }

    public class SeriesData
    {
        /// <summary>
        /// A set of datapoints that describe the data to graph        
        /// String describes the xAxis label, the value is the data point
        /// </summary>
        public Dictionary<string, Double> Data = new Dictionary<string, double>();

        /// <summary>
        /// Text to be used for the legend
        /// </summary>
        public string Legend;

        /// <summary>
        /// Color of the line/bar/graph element. If left unset random color is used
        /// </summary>
        public Color LineColor = Color.Empty;

        /// <summary>
        /// Expression that is used to display the points on the chart
        /// 
        /// Examples:
        /// #VALY     -   data point value
        /// #VALX     -   label value
        /// #VALY{n0} -   data point value with format expression
        /// </summary>
        public string LabelExpression;
    }


}
